local SCA, Ace3, LSM = unpack(select(2,...))
Ace3.version = GetAddOnMetadata(..., 'Version')

--[[---------------------------------------
	CONSTANTS
-----------------------------------------]]
local SOUND_SOURCE_OPTIONS = {
	[0] = "Audio Packs",
	[1] = "SharedMedia",
	[2] = "Custom",
}

local SOUND_CHANNEL_OPTIONS = {
	["Master"] = MASTER,
	["SFX"] = SOUND,
	["Music"] = MUSIC_VOLUME,
	["Ambience"] = AMBIENCE_VOLUME,
	["Dialog"] = DIALOG_VOLUME,
}

local function InitOptions(options,arg1,arg2,arg3)
	local db = Ace3.db.char

	--if (arg2) then
		if (db.audio[arg1][arg2].custom.customType == "file") then
			options.args[arg3].args.CustomSound.args.audioLocation.width = "full"
		else
			options.args[arg3].args.CustomSound.args.audioLocation.width = "double"
		end
	--[[else
		if (db.audio[arg1].custom.customType == "file") then
			options.args[arg3].args.CustomSound.args.audioLocation.width = "full"
		else
			options.args[arg3].args.CustomSound.args.audioLocation.width = "double"
		end
	end]]
end

local function VerifyCustomPath(value,db,trigger,inputType)
	if (strfind(value,".mp3")) then
		value = gsub(value,".mp3",'')
	elseif (strfind(value,".ogg")) then
		value = gsub(value,".ogg",'')
	end
	
	if (strsub(value,#value) ~= [[\]] and db.audio[trigger][inputType].custom.customType == "pack" and #value > 0) then
		value = value..[[\]]
	elseif (strsub(value,#value) == [[\]] and db.audio[trigger][inputType].custom.customType == "file") then
		value = strsub(value,0,#value - 1)
	end
	
	return value
end
--/script local a = { [51505] = "Lava Burst",[188196] = "Lightning Bolt",[188443] = "Chain Lightning"}; for v,i in pairs(a) do print(tostring(v)..", "..tostring(i)) end
--/script local a = { ["Lava Burst"] = true, ["Chain Lightning"] = true,} for v,i in pairs(a) do print("hello") end
local function ReloadCritIgnoreList(options)
	local db = Ace3.db.char
	local args = {}
	local ctr = 1
	
	--table.sort(db.ignore)
	for k,v in pairs(db.ignore) do
		args["spell"..ctr] = {
			order = ctr,
			type = "toggle",
			desc = "Spell ID: "..k,
			descStyle = "inline",
			name = "|cFFEBC000"..v.."|r",
			get = function() 
				return true
			end,
			set = function(self,value)
				db.ignore[k] = nil
				ReloadCritIgnoreList(options)
			end,
		}
		ctr = ctr + 1
	end
	
	options.args.ignoreTab.args.ignoreList.args = args
end


local crit_options

local function GetCritOptions()
	if (not crit_options) then
		--local db = Ace3.db.char
		local db = Ace3.db.char
		
		crit_options = {
			type = "group",
			childGroups = "tab",
			order = 1,
			name = "Crit Sound Options",
			desc = '',
			args = {
				damageTab = {
					name = COMBATLOG_HIGHLIGHT_DAMAGE,
					order = 1,
					type = "group",
					args = {
						source = {
							order = 1,
							type = "select",
							name = "Audio Source",
							desc = "Choose which sound pack you want to play for crit sounds",
							get = function()
								return db.audio.crits.damage.options.source
							end,
							set = function(self,value)										
								db.audio.crits.damage.options.source = value
							end,
							values = SOUND_SOURCE_OPTIONS,
						},
						channel = {
							order = 2,
							type = "select",
							name = "Sound Channel",
							desc = "Choose which channel for the audio to play",
							get = function()
								return db.audio.crits.damage.options.channel
							end,
							set = function(self,value)
								db.audio.crits.damage.options.channel = value
							end,
							values = SOUND_CHANNEL_OPTIONS,
						},
						sOptions = {
							name = "Trigger Options",
							order = 3,
							type = "group",
							guiInline = true,
							args = {
								Damage = {
									order = 1,
									type = "toggle",
									name = "Damage Spells",
									get = function() 
										return db.audio.crits.damage.options.spells
									end,
									set = function(self,value)
										db.audio.crits.damage.options.spells = value
									end,
								},
								DoTs = {
									order = 2,
									type = "toggle",
									name = "DoT Spells",
									get = function() 
										return db.audio.crits.damage.options.dots
									end,
									set = function(self,value)
										db.audio.crits.damage.options.dots = value
									end,
								},
								PetDamage = {
									order = 3,
									type = "toggle",
									name = "Pet Damage Spells",
									get = function() 
										return db.audio.crits.damage.options.pet
									end,
									set = function(self,value)
										db.audio.crits.damage.options.pet = value
									end,
								},
								Threshold = {
									order = 4,
									type = "range",
									name = "Damage Trigger Threshold",
									desc = "Restrict the audio queues to a specific damage threshold",
									min = 0,
									max = 10000000,
									step = 25000,
									bigStep = 25000,
									get = function() 
										return db.audio.crits.damage.options.threshold
									end,
									set = function(self,value)
										db.audio.crits.damage.options.threshold = value
									end,
									width = "full",
								},
							},
						},
						SoundPacks = {
							name = "SCA Sound Packs",
							order = 4,
							type = "group",
							inline = true,
							hidden = function()
								if (db.audio.crits.damage.options.source ~= 0) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								packSelect = {
									order = 1,
									type = "select",
									name = "Sound Pack",
									desc = "Choose which sound pack you want to play for crit sounds",
									descStyle = "inline",
									get = function()
										return db.audio.crits.damage.packs.index
									end,
									set = function(self,value)										
										if (value > 0) then
											local audio = SCA.library[value]
											
											db.audio.crits.damage.packs.index = value
											db.audio.crits.damage.packs.text = audio.text
											db.audio.crits.damage.packs.value = audio.value
											db.audio.crits.damage.packs.path = audio.path
											db.audio.crits.damage.packs.max = audio.max
										end
									end,
									values = function()
										local packs = {}
										packs[0] = "Select Sound Pack"
										for i=1,#SCA.library do
											tinsert(packs,SCA.library[i].text)
										end
										return packs
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						SharedMedia = {
							name = "SharedMedia",
							order = 5,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.crits.damage.options.source ~= 1) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								soundName = {
									order = 1,
									type = "select",
									dialogControl = "LSM30_Sound",
									name = "Select Audio",
									desc = "Select an audio file from the SharedMedia library",
									values = LSM:HashTable("sound"),
									get = function() return db.audio.crits.damage.shared.value end,
									set = function(self,value)
										db.audio.crits.damage.shared.value = value
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						CustomSound = {
							name = "Custom Audio",
							order = 6,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.crits.damage.options.source ~= 2) then
									return true
								else
									return false
								end
							end,
							args = {
								audioSource = {
									order = 1,
									type = "select",
									name = "Audio Type",
									desc = "Choose whether your custom audio is a file or sound pack",
									get = function()
										return db.audio.crits.damage.custom.customType
									end,
									set = function(self,value)
										db.audio.crits.damage.custom.customType = value
										
										if (value == "file") then
											crit_options.args.damageTab.args.CustomSound.args.audioLocation.width = "full"
										else
											crit_options.args.damageTab.args.CustomSound.args.audioLocation.width = "double"
										end
									end,
									values = {
										["file"] = "File",
										["pack"] = "Pack",
									},
								},
								fileType = {
									order = 2,
									type = "select",
									name = "File Type",
									desc = "Choose the file type for your audio files",
									get = function()
										return db.audio.crits.damage.custom.fileType
									end,
									set = function(self,value)
										db.audio.crits.damage.custom.fileType = value
									end,
									values = {
										[".mp3"] = "MP3",
										[".ogg"] = "OGG",
									},
								},
								spacing1 = {
									order = 3,
									type = 'description',
									name = " ",
								},
								audioLocation = {
									type = "input",
									order = 4,				
									name = "Audio Path",
									desc = "Audio file/path location",
									get = function()
											return VerifyCustomPath(db.audio.crits.damage.custom.path,db,'crits','damage')
									end,
									set = function(self,value)								
										db.audio.crits.damage.custom.path = VerifyCustomPath(value,db,'crits','damage')
									end,
								},
								numFiles = {
									type = "input",
									order = 5,
									name = "Number of files",
									desc = "Indicate the amount of files in the sound pack",
									hidden = function()
										if (db.audio.crits.damage.custom.customType == "file") then
											return true
										else
											return false
										end
									end,
									get = function() 
										return db.audio.crits.damage.custom.max
									end,
									set = function(self,value)
										db.audio.crits.damage.custom.max = value
									end,
								},
								test = {
									order = 6,
									type = "execute",
									name = "Test",
									func = function()
										if (db.audio.crits.damage.custom.customType == "pack") then
											if (not PlaySoundFile(db.audio.crits.damage.custom.path..tostring(math.random(1,db.audio.crits.damage.custom.max))..db.audio.crits.damage.custom.fileType,db.audio.crits.damage.options.channel)) then
												crit_options.args.damageTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												crit_options.args.damageTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										else
											if (not PlaySoundFile(db.audio.crits.damage.custom.path..db.audio.crits.damage.custom.fileType,db.audio.crits.damage.options.channel)) then
												crit_options.args.damageTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												crit_options.args.damageTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										end
									end,
								},
								errorMsg = {
									order = 7,
									type = 'description',
									name = " ",
									width = "double",
								},
							},
						},
					},
				},
				healingTab = {
					name = SHOW_COMBAT_HEALING,
					order = 2,
					type = "group",
					args = {
						source = {
							order = 1,
							type = "select",
							name = "Audio Source",
							desc = "Choose which sound pack you want to play for crit sounds",
							get = function()
								return db.audio.crits.heals.options.source
							end,
							set = function(self,value)										
								db.audio.crits.heals.options.source = value
							end,
							values = SOUND_SOURCE_OPTIONS,
						},
						audioSource = {
							order = 2,
							type = "select",
							name = "Sound Channel",
							desc = "Choose which channel for the audio to play",
							get = function()
								return db.audio.crits.heals.options.channel
							end,
							set = function(self,value)
								db.audio.crits.heals.options.channel = value
							end,
							values = SOUND_CHANNEL_OPTIONS,
						},
						sOptions = {
							name = "Sound Options",
							order = 3,
							type = "group",
							guiInline = true,
							args = {
								Healing = {
									order = 1,
									type = "toggle",
									name = "Healing Spells",
									get = function() 
										return db.audio.crits.heals.options.spells
									end,
									set = function(self,value)
										db.audio.crits.heals.options.spells = value
									end,
								},
								HoTs = {
									order = 2,
									type = "toggle",
									name = "Hot Spells",
									get = function() 
										return db.audio.crits.heals.options.hots
									end,
									set = function(self,value)
										db.audio.crits.heals.options.hots = value
									end,
								},
								PetHealing = {
									order = 3,
									type = "toggle",
									name = "Pet Healing Spells",
									get = function() 
										return db.audio.crits.heals.options.pet
									end,
									set = function(self,value)
										db.audio.crits.heals.options.pet = value
									end,
								},
								Threshold = {
									order = 4,
									type = "range",
									name = "Healing Trigger Threshold",
									desc = "Restrict the audio queues to a specific healing threshold",
									min = 0,
									max = 10000000,
									step = 25000,
									bigStep = 25000,
									get = function() 
										return db.audio.crits.heals.options.threshold
									end,
									set = function(self,value)
										db.audio.crits.heals.options.threshold = value
									end,
									width = "full",
								},
							},
						},
						SoundPacks = {
							name = "SCA Sound Packs",
							order = 4,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.crits.heals.options.source ~= 0) then
									return true
								else
									return false
								end
							end,
							args = {
								packSelect = {
									order = 1,
									type = "select",
									name = "Sound Pack",
									desc = "Choose which sound pack you want to play for crit sounds",
									get = function()
										return db.audio.crits.heals.packs.index
									end,
									set = function(self,value)
										if (value > 0) then
											local audio = SCA.library[value]
											
											db.audio.crits.heals.packs.index = value
											db.audio.crits.heals.packs.text = audio.text
											db.audio.crits.heals.packs.value = audio.value
											db.audio.crits.heals.packs.path = audio.path
											db.audio.crits.heals.packs.max = audio.max
										end
									end,
									values = function()
										local packs = {}
										packs[0] = "Select Sound Pack"
										for i=1,#SCA.library do
											tinsert(packs,SCA.library[i].text)
										end
										return packs
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						SharedMedia = {
							name = "SharedMedia",
							order = 5,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.crits.heals.options.source ~= 1) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								soundName = {
									order = 1,
									type = "select",
									dialogControl = "LSM30_Sound",
									name = "Select Audio",
									desc = "Select an audio file from the SharedMedia library",
									values = LSM:HashTable("sound"),
									get = function() return db.audio.crits.heals.shared.value end,
									set = function(self,value)
										db.audio.crits.heals.shared.value = value
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						CustomSound = {
							name = "Custom Audio",
							order = 6,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.crits.heals.options.source ~= 2) then
									return true
								else
									return false
								end
							end,
							args = {
								audioSource = {
									order = 1,
									type = "select",
									name = "Audio Type",
									desc = "Choose whether your custom audio is a file or sound pack",
									get = function()
										return db.audio.crits.heals.custom.customType
									end,
									set = function(self,value)
										db.audio.crits.heals.custom.customType = value
										
										if (value == "file") then
											crit_options.args.healingTab.args.CustomSound.args.audioLocation.width = "full"
										else
											crit_options.args.healingTab.args.CustomSound.args.audioLocation.width = "double"
										end
									end,
									values = {
										["file"] = "File",
										["pack"] = "Pack",
									},
								},
								fileType = {
									order = 2,
									type = "select",
									name = "File Type",
									desc = "Choose the file type for your audio files",
									get = function()
										return db.audio.crits.heals.custom.fileType
									end,
									set = function(self,value)
										db.audio.crits.heals.custom.fileType = value
									end,
									values = {
										[".mp3"] = "MP3",
										[".ogg"] = "OGG",
									},
								},
								spacing1 = {
									order = 3,
									type = 'description',
									name = " ",
								},
								audioLocation = {
									type = "input",
									order = 4,				
									name = "Audio Path",
									desc = "Audio file/path location",
									get = function()
											return VerifyCustomPath(db.audio.crits.heals.custom.path,db,'crits','heals')
									end,
									set = function(self,value)								
										db.audio.crits.heals.custom.path = VerifyCustomPath(value,db,'crits','heals')
									end,
								},
								numFiles = {
									type = "input",
									order = 5,
									name = "Number of files",
									desc = "Indicate the amount of files in the sound pack",
									hidden = function()
										if (db.audio.crits.heals.custom.customType == "file") then
											return true
										else
											return false
										end
									end,
									get = function() 
										return db.audio.crits.heals.custom.max
									end,
									set = function(self,value)
										db.audio.crits.heals.custom.max = value
									end,
								},
								test = {
									order = 6,
									type = "execute",
									name = "Test",
									func = function()
										if (db.audio.crits.heals.custom.customType == "pack") then
											if (not PlaySoundFile(db.audio.crits.heals.custom.path..tostring(math.random(1,db.audio.crits.heals.custom.max))..db.audio.crits.heals.custom.fileType,db.audio.crits.heals.options.channel)) then
												crit_options.args.healingTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												crit_options.args.healingTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										else
											if (not PlaySoundFile(db.audio.crits.heals.custom.path..db.audio.crits.heals.custom.fileType,db.audio.crits.heals.options.channel)) then
												crit_options.args.healingTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												crit_options.args.healingTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										end
									end,
								},
								errorMsg = {
									order = 7,
									type = 'description',
									name = " ",
									width = "double",
								},
							},
						},
					},
				},
				ignoreTab = {
					name = IGNORE,
					order = 2,
					type = "group",
					args = {
						input = {
							name = "Spell Input",
							order = 0,
							type = "group",
							desc = "Click on a checkbox to unignore a spell",
							guiInline = true,
							args = {
								spellInfo = {
									type = "input",
									order = 1,				
									name = "Spell ID / Name",
									desc = "Input spell id, or name, to be ignored by the addon.\n\n|cFFFF0000NOTE|r: Spell IDs are more reliable. Downloading the addon \"|cFF00FF00idTip|r\" shows Spell IDs on tooltips.\n\nIf entering a spell's name results in an error, you will have to use the spell's ID",
									get = function()
											return SCA.temp.ignore
									end,
									set = function(self,value)
										local name,_,_,_,_,_,spellID = GetSpellInfo(value)
										
										if (name and not db.ignore[spellID] and IsUsableSpell(name)) then
											crit_options.args.ignoreTab.args.input.args.submit.disabled = false
											crit_options.args.ignoreTab.args.input.args.errorMsg.name = " "
											SCA.temp.ignore = value
										else
											crit_options.args.ignoreTab.args.input.args.submit.disabled = true
											if (db.ignore[spellID]) then
												crit_options.args.ignoreTab.args.input.args.errorMsg.name = "|cFFFF0000Spell is already ignored|r"
											elseif (name and not IsUsableSpell(name)) then
												crit_options.args.ignoreTab.args.input.args.errorMsg.name = "|cFFFF0000You cannot use that spell|r"
											else
												crit_options.args.ignoreTab.args.input.args.errorMsg.name = "|cFFFF0000Spell not found|r"
											end
											
											SCA.temp.ignore = ''
										end
										
									end,
								},
								submit = {
									order = 2,
									type = "execute",
									name = "Ignore",
									disabled = true,
									func = function()
										local name,_,_,_,_,_,spellID = GetSpellInfo(SCA.temp.ignore)
										
										--if (name) then
											db.ignore[spellID] = name
											crit_options.args.ignoreTab.args.input.args.submit.disabled = true
											SCA.temp.ignore = ''
											--ignore_options.args.input.args.errorMsg.name = ""
										--else
											--ignore_options.args.input.args.errorMsg.name = "|cFFFF0000Spell not found|r"
										--end
										ReloadCritIgnoreList(GetCritOptions())
									end,
								},
								errorMsg = {
									order = 3,
									type = 'description',
									name = " ",
								},
							},
						},
						ignoreList = {
							name = "Ignored Spells",
							order = 1,
							type = "group",
							dialogInline = true,
							args = {
							},
						},
					},
				},
			},
		}
	end
	
	return crit_options
end

local defense_options

local function GetDefenseOptions()
	if (not defense_options) then
		local db = Ace3.db.char
		
		defense_options = {
			type = "group",
			childGroups = "tab",
			order = 1,
			name = "Defense Audio Options",
			desc = '',
			args = {
				dodgeTab = {
					name = COMBAT_TEXT_DODGE,
					order = 1,
					type = "group",
					args = {
						toggle = {
							order = 1,
							type = "toggle",
							name = "Enable",
							get = function() 
								return db.audio.defense.dodge.options.enabled
							end,
							set = function(self,value)
								db.audio.defense.dodge.options.enabled = value
							end,
						},
						source = {
							order = 2,
							type = "select",
							name = "Audio Source",
							desc = "Choose which sound pack you want to play for crit sounds",
							get = function()
								return db.audio.defense.dodge.options.source
							end,
							set = function(self,value)										
								db.audio.defense.dodge.options.source = value
							end,
							values = SOUND_SOURCE_OPTIONS,
						},
						channel = {
							order = 3,
							type = "select",
							name = "Sound Channel",
							desc = "Choose which channel for the audio to play",
							get = function()
								return db.audio.defense.dodge.options.channel
							end,
							set = function(self,value)
								db.audio.defense.dodge.options.channel = value
							end,
							values = SOUND_CHANNEL_OPTIONS,
						},
						SoundPacks = {
							name = "SCA Sound Packs",
							order = 4,
							type = "group",
							inline = true,
							hidden = function()
								if (db.audio.defense.dodge.options.source ~= 0) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								packSelect = {
									order = 1,
									type = "select",
									name = "Sound Pack",
									desc = "Choose which sound pack you want to play for crit sounds",
									descStyle = "inline",
									get = function()
										return db.audio.defense.dodge.packs.index
									end,
									set = function(self,value)										
										if (value > 0) then
											local audio = SCA.library[value]
											
											db.audio.defense.dodge.packs.index = value
											db.audio.defense.dodge.packs.text = audio.text
											db.audio.defense.dodge.packs.value = audio.value
											db.audio.defense.dodge.packs.path = audio.path
											db.audio.defense.dodge.packs.max = audio.max
										end
									end,
									values = function()
										local packs = {}
										packs[0] = "Select Sound Pack"
										for i=1,#SCA.library do
											tinsert(packs,SCA.library[i].text)
										end
										return packs
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						SharedMedia = {
							name = "SharedMedia",
							order = 5,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.defense.dodge.options.source ~= 1) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								soundName = {
									order = 1,
									type = "select",
									dialogControl = "LSM30_Sound",
									name = "Select Audio",
									desc = "Select an audio file from the SharedMedia library",
									values = LSM:HashTable("sound"),
									get = function() return db.audio.defense.dodge.shared.value end,
									set = function(self,value)
										db.audio.defense.dodge.shared.value = value
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						CustomSound = {
							name = "Custom Audio",
							order = 6,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.defense.dodge.options.source ~= 2) then
									return true
								else
									return false
								end
							end,
							args = {
								audioSource = {
									order = 1,
									type = "select",
									name = "Audio Type",
									desc = "Choose whether your custom audio is a file or sound pack",
									get = function()
										return db.audio.defense.dodge.custom.customType
									end,
									set = function(self,value)
										db.audio.defense.dodge.custom.customType = value
										
										if (value == "file") then
											defense_options.args.dodgeTab.args.CustomSound.args.audioLocation.width = "full"
										else
											defense_options.args.dodgeTab.args.CustomSound.args.audioLocation.width = "double"
										end
									end,
									values = {
										["file"] = "File",
										["pack"] = "Pack",
									},
								},
								fileType = {
									order = 2,
									type = "select",
									name = "File Type",
									desc = "Choose the file type for your audio files",
									get = function()
										return db.audio.defense.dodge.custom.fileType
									end,
									set = function(self,value)
										db.audio.defense.dodge.custom.fileType = value
									end,
									values = {
										[".mp3"] = "MP3",
										[".ogg"] = "OGG",
									},
								},
								spacing1 = {
									order = 3,
									type = 'description',
									name = " ",
								},
								audioLocation = {
									type = "input",
									order = 4,				
									name = "Audio Path",
									desc = "Audio file/path location",
									get = function()
											return VerifyCustomPath(db.audio.defense.dodge.custom.path,db,'crits','damage')
									end,
									set = function(self,value)								
										db.audio.defense.dodge.custom.path = VerifyCustomPath(value,db,'crits','damage')
									end,
								},
								numFiles = {
									type = "input",
									order = 5,
									name = "Number of files",
									desc = "Indicate the amount of files in the sound pack",
									hidden = function()
										if (db.audio.defense.dodge.custom.customType == "file") then
											return true
										else
											return false
										end
									end,
									get = function() 
										return db.audio.defense.dodge.custom.max
									end,
									set = function(self,value)
										db.audio.defense.dodge.custom.max = value
									end,
								},
								test = {
									order = 6,
									type = "execute",
									name = "Test",
									func = function()
										if (db.audio.defense.dodge.custom.customType == "pack") then
											if (not PlaySoundFile(db.audio.defense.dodge.custom.path..tostring(math.random(1,db.audio.defense.dodge.custom.max))..db.audio.defense.dodge.custom.fileType,db.audio.defense.dodge.options.channel)) then
												defense_options.args.dodgeTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												defense_options.args.dodgeTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										else
											if (not PlaySoundFile(db.audio.defense.dodge.custom.path..db.audio.defense.dodge.custom.fileType,db.audio.defense.dodge.options.channel)) then
												defense_options.args.dodgeTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												defense_options.args.dodgeTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										end
									end,
								},
								errorMsg = {
									order = 7,
									type = 'description',
									name = " ",
									width = "double",
								},
							},
						},
					},
				},
				parryTab = {
					name = COMBAT_TEXT_PARRY,
					order = 1,
					type = "group",
					args = {
						toggle = {
							order = 1,
							type = "toggle",
							name = "Enable",
							get = function() 
								return db.audio.defense.parry.options.enabled
							end,
							set = function(self,value)
								db.audio.defense.parry.options.enabled = value
							end,
						},
						source = {
							order = 2,
							type = "select",
							name = "Audio Source",
							desc = "Choose which sound pack you want to play for crit sounds",
							get = function()
								return db.audio.defense.parry.options.source
							end,
							set = function(self,value)										
								db.audio.defense.parry.options.source = value
							end,
							values = SOUND_SOURCE_OPTIONS,
						},
						channel = {
							order = 3,
							type = "select",
							name = "Sound Channel",
							desc = "Choose which channel for the audio to play",
							get = function()
								return db.audio.defense.parry.options.channel
							end,
							set = function(self,value)
								db.audio.defense.parry.options.channel = value
							end,
							values = SOUND_CHANNEL_OPTIONS,
						},
						SoundPacks = {
							name = "SCA Sound Packs",
							order = 4,
							type = "group",
							inline = true,
							hidden = function()
								if (db.audio.defense.parry.options.source ~= 0) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								packSelect = {
									order = 1,
									type = "select",
									name = "Sound Pack",
									desc = "Choose which sound pack you want to play for crit sounds",
									descStyle = "inline",
									get = function()
										return db.audio.defense.parry.packs.index
									end,
									set = function(self,value)										
										if (value > 0) then
											local audio = SCA.library[value]
											
											db.audio.defense.parry.packs.index = value
											db.audio.defense.parry.packs.text = audio.text
											db.audio.defense.parry.packs.value = audio.value
											db.audio.defense.parry.packs.path = audio.path
											db.audio.defense.parry.packs.max = audio.max
										end
									end,
									values = function()
										local packs = {}
										packs[0] = "Select Sound Pack"
										for i=1,#SCA.library do
											tinsert(packs,SCA.library[i].text)
										end
										return packs
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						SharedMedia = {
							name = "SharedMedia",
							order = 5,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.defense.parry.options.source ~= 1) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								soundName = {
									order = 1,
									type = "select",
									dialogControl = "LSM30_Sound",
									name = "Select Audio",
									desc = "Select an audio file from the SharedMedia library",
									values = LSM:HashTable("sound"),
									get = function() return db.audio.defense.parry.shared.value end,
									set = function(self,value)
										db.audio.defense.parry.shared.value = value
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						CustomSound = {
							name = "Custom Audio",
							order = 6,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.defense.parry.options.source ~= 2) then
									return true
								else
									return false
								end
							end,
							args = {
								audioSource = {
									order = 1,
									type = "select",
									name = "Audio Type",
									desc = "Choose whether your custom audio is a file or sound pack",
									get = function()
										return db.audio.defense.parry.custom.customType
									end,
									set = function(self,value)
										db.audio.defense.parry.custom.customType = value
										
										if (value == "file") then
											defense_options.args.parryTab.args.CustomSound.args.audioLocation.width = "full"
										else
											defense_options.args.parryTab.args.CustomSound.args.audioLocation.width = "double"
										end
									end,
									values = {
										["file"] = "File",
										["pack"] = "Pack",
									},
								},
								fileType = {
									order = 2,
									type = "select",
									name = "File Type",
									desc = "Choose the file type for your audio files",
									get = function()
										return db.audio.defense.parry.custom.fileType
									end,
									set = function(self,value)
										db.audio.defense.parry.custom.fileType = value
									end,
									values = {
										[".mp3"] = "MP3",
										[".ogg"] = "OGG",
									},
								},
								spacing1 = {
									order = 3,
									type = 'description',
									name = " ",
								},
								audioLocation = {
									type = "input",
									order = 4,				
									name = "Audio Path",
									desc = "Audio file/path location",
									get = function()
											return VerifyCustomPath(db.audio.defense.parry.custom.path,db,'crits','damage')
									end,
									set = function(self,value)								
										db.audio.defense.parry.custom.path = VerifyCustomPath(value,db,'crits','damage')
									end,
								},
								numFiles = {
									type = "input",
									order = 5,
									name = "Number of files",
									desc = "Indicate the amount of files in the sound pack",
									hidden = function()
										if (db.audio.defense.parry.custom.customType == "file") then
											return true
										else
											return false
										end
									end,
									get = function() 
										return db.audio.defense.parry.custom.max
									end,
									set = function(self,value)
										db.audio.defense.parry.custom.max = value
									end,
								},
								test = {
									order = 6,
									type = "execute",
									name = "Test",
									func = function()
										if (db.audio.defense.parry.custom.customType == "pack") then
											if (not PlaySoundFile(db.audio.defense.parry.custom.path..tostring(math.random(1,db.audio.defense.parry.custom.max))..db.audio.defense.parry.custom.fileType,db.audio.defense.parry.options.channel)) then
												defense_options.args.parryTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												defense_options.args.parryTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										else
											if (not PlaySoundFile(db.audio.defense.parry.custom.path..db.audio.defense.parry.custom.fileType,db.audio.defense.parry.options.channel)) then
												defense_options.args.parryTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												defense_options.args.parryTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										end
									end,
								},
								errorMsg = {
									order = 7,
									type = 'description',
									name = " ",
									width = "double",
								},
							},
						},
					},
				},
				blockTab = {
					name = COMBAT_TEXT_BLOCK,
					order = 1,
					type = "group",
					args = {
						toggle = {
							order = 1,
							type = "toggle",
							name = "Enable",
							get = function() 
								return db.audio.defense.block.options.enabled
							end,
							set = function(self,value)
								db.audio.defense.block.options.enabled = value
							end,
						},
						source = {
							order = 2,
							type = "select",
							name = "Audio Source",
							desc = "Choose which sound pack you want to play for crit sounds",
							get = function()
								return db.audio.defense.block.options.source
							end,
							set = function(self,value)										
								db.audio.defense.block.options.source = value
							end,
							values = SOUND_SOURCE_OPTIONS,
						},
						channel = {
							order = 3,
							type = "select",
							name = "Sound Channel",
							desc = "Choose which channel for the audio to play",
							get = function()
								return db.audio.defense.block.options.channel
							end,
							set = function(self,value)
								db.audio.defense.block.options.channel = value
							end,
							values = SOUND_CHANNEL_OPTIONS,
						},
						SoundPacks = {
							name = "SCA Sound Packs",
							order = 4,
							type = "group",
							inline = true,
							hidden = function()
								if (db.audio.defense.block.options.source ~= 0) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								packSelect = {
									order = 1,
									type = "select",
									name = "Sound Pack",
									desc = "Choose which sound pack you want to play for crit sounds",
									descStyle = "inline",
									get = function()
										return db.audio.defense.block.packs.index
									end,
									set = function(self,value)										
										if (value > 0) then
											local audio = SCA.library[value]
											
											db.audio.defense.block.packs.index = value
											db.audio.defense.block.packs.text = audio.text
											db.audio.defense.block.packs.value = audio.value
											db.audio.defense.block.packs.path = audio.path
											db.audio.defense.block.packs.max = audio.max
										end
									end,
									values = function()
										local packs = {}
										packs[0] = "Select Sound Pack"
										for i=1,#SCA.library do
											tinsert(packs,SCA.library[i].text)
										end
										return packs
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						SharedMedia = {
							name = "SharedMedia",
							order = 5,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.defense.block.options.source ~= 1) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								soundName = {
									order = 1,
									type = "select",
									dialogControl = "LSM30_Sound",
									name = "Select Audio",
									desc = "Select an audio file from the SharedMedia library",
									values = LSM:HashTable("sound"),
									get = function() return db.audio.defense.block.shared.value end,
									set = function(self,value)
										db.audio.defense.block.shared.value = value
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						CustomSound = {
							name = "Custom Audio",
							order = 6,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.defense.block.options.source ~= 2) then
									return true
								else
									return false
								end
							end,
							args = {
								audioSource = {
									order = 1,
									type = "select",
									name = "Audio Type",
									desc = "Choose whether your custom audio is a file or sound pack",
									get = function()
										return db.audio.defense.block.custom.customType
									end,
									set = function(self,value)
										db.audio.defense.block.custom.customType = value
										
										if (value == "file") then
											defense_options.args.blockTab.args.CustomSound.args.audioLocation.width = "full"
										else
											defense_options.args.blockTab.args.CustomSound.args.audioLocation.width = "double"
										end
									end,
									values = {
										["file"] = "File",
										["pack"] = "Pack",
									},
								},
								fileType = {
									order = 2,
									type = "select",
									name = "File Type",
									desc = "Choose the file type for your audio files",
									get = function()
										return db.audio.defense.block.custom.fileType
									end,
									set = function(self,value)
										db.audio.defense.block.custom.fileType = value
									end,
									values = {
										[".mp3"] = "MP3",
										[".ogg"] = "OGG",
									},
								},
								spacing1 = {
									order = 3,
									type = 'description',
									name = " ",
								},
								audioLocation = {
									type = "input",
									order = 4,				
									name = "Audio Path",
									desc = "Audio file/path location",
									get = function()
											return VerifyCustomPath(db.audio.defense.block.custom.path,db,'crits','damage')
									end,
									set = function(self,value)								
										db.audio.defense.block.custom.path = VerifyCustomPath(value,db,'crits','damage')
									end,
								},
								numFiles = {
									type = "input",
									order = 5,
									name = "Number of files",
									desc = "Indicate the amount of files in the sound pack",
									hidden = function()
										if (db.audio.defense.block.custom.customType == "file") then
											return true
										else
											return false
										end
									end,
									get = function() 
										return db.audio.defense.block.custom.max
									end,
									set = function(self,value)
										db.audio.defense.block.custom.max = value
									end,
								},
								test = {
									order = 6,
									type = "execute",
									name = "Test",
									func = function()
										if (db.audio.defense.block.custom.customType == "pack") then
											if (not PlaySoundFile(db.audio.defense.block.custom.path..tostring(math.random(1,db.audio.defense.block.custom.max))..db.audio.defense.block.custom.fileType,db.audio.defense.block.options.channel)) then
												defense_options.args.blockTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												defense_options.args.blockTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										else
											if (not PlaySoundFile(db.audio.defense.block.custom.path..db.audio.defense.block.custom.fileType,db.audio.defense.block.options.channel)) then
												defense_options.args.blockTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												defense_options.args.blockTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										end
									end,
								},
								errorMsg = {
									order = 7,
									type = 'description',
									name = " ",
									width = "double",
								},
							},
						},
					},
				},
				absorbTab = {
					name = COMBAT_TEXT_ABSORB,
					order = 1,
					type = "group",
					args = {
						toggle = {
							order = 1,
							type = "toggle",
							name = "Enable",
							get = function() 
								return db.audio.defense.absorb.options.enabled
							end,
							set = function(self,value)
								db.audio.defense.absorb.options.enabled = value
							end,
						},
						source = {
							order = 2,
							type = "select",
							name = "Audio Source",
							desc = "Choose which sound pack you want to play for crit sounds",
							get = function()
								return db.audio.defense.absorb.options.source
							end,
							set = function(self,value)										
								db.audio.defense.absorb.options.source = value
							end,
							values = SOUND_SOURCE_OPTIONS,
						},
						channel = {
							order = 3,
							type = "select",
							name = "Sound Channel",
							desc = "Choose which channel for the audio to play",
							get = function()
								return db.audio.defense.absorb.options.channel
							end,
							set = function(self,value)
								db.audio.defense.absorb.options.channel = value
							end,
							values = SOUND_CHANNEL_OPTIONS,
						},
						SoundPacks = {
							name = "SCA Sound Packs",
							order = 4,
							type = "group",
							inline = true,
							hidden = function()
								if (db.audio.defense.absorb.options.source ~= 0) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								packSelect = {
									order = 1,
									type = "select",
									name = "Sound Pack",
									desc = "Choose which sound pack you want to play for crit sounds",
									descStyle = "inline",
									get = function()
										return db.audio.defense.absorb.packs.index
									end,
									set = function(self,value)										
										if (value > 0) then
											local audio = SCA.library[value]
											
											db.audio.defense.absorb.packs.index = value
											db.audio.defense.absorb.packs.text = audio.text
											db.audio.defense.absorb.packs.value = audio.value
											db.audio.defense.absorb.packs.path = audio.path
											db.audio.defense.absorb.packs.max = audio.max
										end
									end,
									values = function()
										local packs = {}
										packs[0] = "Select Sound Pack"
										for i=1,#SCA.library do
											tinsert(packs,SCA.library[i].text)
										end
										return packs
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						SharedMedia = {
							name = "SharedMedia",
							order = 5,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.defense.absorb.options.source ~= 1) then
									return true
								else
									return false
								end
							end,
							width = "half",
							args = {
								soundName = {
									order = 1,
									type = "select",
									dialogControl = "LSM30_Sound",
									name = "Select Audio",
									desc = "Select an audio file from the SharedMedia library",
									values = LSM:HashTable("sound"),
									get = function() return db.audio.defense.absorb.shared.value end,
									set = function(self,value)
										db.audio.defense.absorb.shared.value = value
									end,
								},
								spacing = {
									order = 2,
									type = 'description',
									name = " ",
									width = "full",
								},
							},
						},
						CustomSound = {
							name = "Custom Audio",
							order = 6,
							type = "group",
							guiInline = true,
							hidden = function()
								if (db.audio.defense.absorb.options.source ~= 2) then
									return true
								else
									return false
								end
							end,
							args = {
								audioSource = {
									order = 1,
									type = "select",
									name = "Audio Type",
									desc = "Choose whether your custom audio is a file or sound pack",
									get = function()
										return db.audio.defense.absorb.custom.customType
									end,
									set = function(self,value)
										db.audio.defense.absorb.custom.customType = value
										
										if (value == "file") then
											defense_options.args.absorbTab.args.CustomSound.args.audioLocation.width = "full"
										else
											defense_options.args.absorbTab.args.CustomSound.args.audioLocation.width = "double"
										end
									end,
									values = {
										["file"] = "File",
										["pack"] = "Pack",
									},
								},
								fileType = {
									order = 2,
									type = "select",
									name = "File Type",
									desc = "Choose the file type for your audio files",
									get = function()
										return db.audio.defense.absorb.custom.fileType
									end,
									set = function(self,value)
										db.audio.defense.absorb.custom.fileType = value
									end,
									values = {
										[".mp3"] = "MP3",
										[".ogg"] = "OGG",
									},
								},
								spacing1 = {
									order = 3,
									type = 'description',
									name = " ",
								},
								audioLocation = {
									type = "input",
									order = 4,				
									name = "Audio Path",
									desc = "Audio file/path location",
									get = function()
											return VerifyCustomPath(db.audio.defense.absorb.custom.path,db,'crits','damage')
									end,
									set = function(self,value)								
										db.audio.defense.absorb.custom.path = VerifyCustomPath(value,db,'crits','damage')
									end,
								},
								numFiles = {
									type = "input",
									order = 5,
									name = "Number of files",
									desc = "Indicate the amount of files in the sound pack",
									hidden = function()
										if (db.audio.defense.absorb.custom.customType == "file") then
											return true
										else
											return false
										end
									end,
									get = function() 
										return db.audio.defense.absorb.custom.max
									end,
									set = function(self,value)
										db.audio.defense.absorb.custom.max = value
									end,
								},
								test = {
									order = 6,
									type = "execute",
									name = "Test",
									func = function()
										if (db.audio.defense.absorb.custom.customType == "pack") then
											if (not PlaySoundFile(db.audio.defense.absorb.custom.path..tostring(math.random(1,db.audio.defense.absorb.custom.max))..db.audio.defense.absorb.custom.fileType,db.audio.defense.absorb.options.channel)) then
												defense_options.args.absorbTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												defense_options.args.absorbTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										else
											if (not PlaySoundFile(db.audio.defense.absorb.custom.path..db.audio.defense.absorb.custom.fileType,db.audio.defense.absorb.options.channel)) then
												defense_options.args.absorbTab.args.CustomSound.args.errorMsg.name = "|cFFFF0000Audio file not found!|r"
											else
												defense_options.args.absorbTab.args.CustomSound.args.errorMsg.name = "|cFF00FF00Audio file found!|r"
											end
										end
									end,
								},
								errorMsg = {
									order = 7,
									type = 'description',
									name = " ",
									width = "double",
								},
							},
						},
					},
				},
			},
		}
		
	end
	
	return defense_options
end

function Ace3:SetupOptions()
	local ACFG = LibStub("AceConfig-3.0")
	ACFG:RegisterOptionsTable("CustomAudio Crit Sounds", GetCritOptions)
	ACFG:RegisterOptionsTable("CustomAudio Defense Sounds", GetDefenseOptions)

	local ACD = LibStub("AceConfigDialog-3.0")
	ACD:AddToBlizOptions("CustomAudio Crit Sounds", "Crit Sounds", "CustomAudio")
	ACD:AddToBlizOptions("CustomAudio Defense Sounds", "Defense Sounds", "CustomAudio")
end

local function UpdateInterfaceSettings()
	InitOptions(GetCritOptions(),'crits','damage','damageTab')
	InitOptions(GetCritOptions(),'crits','heals','healingTab')
	InitOptions(GetDefenseOptions(),'defense','dodge','dodgeTab')
	InitOptions(GetDefenseOptions(),'defense','parry','dodgeTab')
	InitOptions(GetDefenseOptions(),'defense','block','dodgeTab')
	InitOptions(GetDefenseOptions(),'defense','absorb','dodgeTab')

	ReloadCritIgnoreList(GetCritOptions())
end

InterfaceOptionsFrame:HookScript("OnShow",function(self)
	UpdateInterfaceSettings()
end)